###################################################################################################
################################     PARAMETER/FUNCTION DEFINE     ################################
###################################################################################################
modelSelection<-function(m,e,g){
  library(bnlearn)
  learning.test=data.frame(M=m,E=e,G=g)
  causal = empty.graph(names(learning.test))
  modelstring(causal ) = "[M][E|M][G|E]"
  
  composite = empty.graph(names(learning.test))
  modelstring(composite) = "[M][E|M][G|M:E]"
  
  Co_responsive = empty.graph(names(learning.test))
  modelstring(Co_responsive) = "[M][E|M][G|M]"
  
  Poised = empty.graph(names(learning.test))
  modelstring(Poised) = "[M][E|M][G]"
  Direct = empty.graph(names(learning.test))
  modelstring(Direct) = "[M][G|M][E]"
  
  coordinate = empty.graph(names(learning.test))
  modelstring(coordinate) = "[M][E][G|M:E]"
  
  AICs <- c(AIC_Causal=score(causal, learning.test,type="aic-g"),
            AIC_Composite=score(composite, learning.test,type="aic-g"),
            AIC_Co_responsive=score(Co_responsive, learning.test,type="aic-g"),
            AIC_Poised=score(Poised, learning.test,type="aic-g"),
            AIC_Direct=score(Direct, learning.test,type="aic-g"),
            AIC_Coordinate=score(coordinate, learning.test,type="aic-g"))
  AIC.min <- min(AICs);
  dAIC=AICs-AIC.min
  names(dAIC)=gsub("AIC_","dAIC_",names(AICs))
  index<- -0.5*dAIC
  w<-exp(index)/sum(exp(index))
  names(w)=gsub("AIC_","weight_",names(AICs))
  model=gsub("weight_","",names(which.max(w)))
  
  return(c(model=model,AICs,dAIC,w,
           ME_mi=ci.test(m,e,test = "mi-g")$statistic,
           ME_p=ci.test(m,e,test = "mi-g")$p.value,
           MG_mi=ci.test(m,g,test = "mi-g")$statistic,
           ME_p=ci.test(m,e,test = "mi-g")$p.value,
           EG_mi=ci.test(e,g,test = "mi-g")$statistic,
           EG_p=ci.test(e,g,test = "mi-g")$p.value,
           CIT_ME_mi=ci.test(m,e,g,test = "mi-g")$statistic,
           CIT_ME_p=ci.test(m,e,g,test = "mi-g")$p.value,
           CIT_MG_mi=ci.test(m,g,e,test = "mi-g")$statistic,
           CIT_MG_p=ci.test(m,g,e,test = "mi-g")$p.value,
           CIT_EG_mi=ci.test(e,g,m,test = "mi-g")$statistic,
           CIT_EG_p=ci.test(e,g,m,test = "mi-g")$p.value))
}
simulate<-function(x,r){
  r*x+sqrt(1-r*r)*rnorm(length(x))
}
pproc<-function(gd,rho,main,filename,width,height,control){
  pdf(filename,width=width,height = height)
  col=c(colorRampPalette(c("#43C6AC","#F8FFAE"))(9),
        "gray",
        colorRampPalette(c("#FFB88C","#DE6262"))(9))
  pos<-rev(seq(0,60,3))
  gd1<-gd[which(gd$r==rho[1]),]
  rocobj<- plot.roc(gd1$type,gd1$weight,percent=TRUE, main=main,
                    print.auc.x=40,print.auc.y=pos[i],ci=T,lwd=2,
            grid=TRUE, col=col[1])
  
  r<-rho[-1]
  pos<-pos[-1]
  col<-col[-1]
  for (i in 1:length(r)) {
    gd2<-gd[which(gd$r==r[i]),]
    roc2 <- plot.roc(gd2$type,gd2$weight, add=T,ci=T,lwd=2,
                     percent=TRUE, main=main,  col=col[i])
  }
  dev.off()
}

###################################################################################################
#####################################     simulated Poised     ####################################
###################################################################################################
library(parallel)
cl<-makeCluster(3)
simu2<-parLapply(cl,seq(-0.9,0.9,0.1),
                          function(r,simulate,modelSelection){
                            lapply(1:1000,function(x){
                              m<-rnorm(100)
                              e<-simulate(x=m,r=r)
                              g<-rnorm(100)
                              model1<-c(modelSelection(m,e,g),r=r,type="simulate")
                              m<-rnorm(100)
                              e<-rnorm(100)
                              g<-rnorm(100)
                              model2<-c(modelSelection(m,e,g),r=r,type="control")
                              rbind(model1,model2)
                            })
                          },simulate,modelSelection)
rdata<-c()
for (i in 1:length(simu2)) {
  x<-do.call(rbind,simu2[[i]])
  rdata<-rbind(rdata,x)
}
rdata=as.data.frame(rdata)
colnames(rdata)[14:19]<-gsub("AIC","weight",colnames(rdata)[14:19])
rdata=rdata[,c("type","weight_Poised","r")]
gd<-rdata
colnames(gd)<-c("type","weight","r")
gd$weight=as.numeric(as.character(gd$weight))
library(pROC)
pproc(gd,rho=unique(gd$r),main="Poised",filename = "Poised_ROC.pdf",
      width = 4,height = 4)
###################################################################################################
#####################################     simulated Direct     ####################################
###################################################################################################
simu3<-parLapply(cl,seq(-0.9,0.9,0.1),
                 function(r,simulate,modelSelection){
                   lapply(1:1000,function(x){
                     m<-rnorm(100)
                     e<-rnorm(100)
                     g<-simulate(x=m,r=r)
                     model1<-c(modelSelection(m,e,g),r=r,type="simulate")
                     m<-rnorm(100)
                     e<-rnorm(100)
                     g<-rnorm(100)
                     model2<-c(modelSelection(m,e,g),r=r,type="control")
                     rbind(model1,model2)
                   })
                 },simulate,modelSelection)
rdata<-c()
for (i in 1:length(simu3)) {
  x<-do.call(rbind,simu3[[i]])
  rdata<-rbind(rdata,x)
}
rdata=as.data.frame(rdata)
colnames(rdata)[14:19]<-gsub("AIC","weight",colnames(rdata)[14:19])
rdata=rdata[,c("type","weight_Direct","r")]
gd<-rdata
colnames(gd)<-c("type","weight","r")
gd$weight=as.numeric(as.character(gd$weight))
library(pROC)
pproc(gd,rho=unique(gd$r),main="Direct",filename = "Direct_ROC.pdf",
      width = 4,height = 4)
###################################################################################################
##############################     simulated Co_responsive     ####################################
###################################################################################################
simuCo_responsive<-parLapply(cl,seq(-0.9,0.9,0.1),
                           function(r,simulate,modelSelection){
  lapply(1:1000,function(x){
    m<-rnorm(100)
    e<-simulate(x=m,r=r)
    g<-simulate(x=m,r=r)
    model1<-c(modelSelection(m,e,g),r=r,type="simulate")
    m<-rnorm(100)
    e<-rnorm(100)
    g<-rnorm(100)
    model2<-c(modelSelection(m,e,g),r=r,type="control")
    rbind(model1,model2)
  })
},simulate,modelSelection)

rdata<-c()
for (i in 1:length(simuCo_responsive)) {
  x<-do.call(rbind,simuCo_responsive[[i]])
  rdata<-rbind(rdata,x)
}
rdata=as.data.frame(rdata)
colnames(rdata)[14:19]<-gsub("AIC","weight",colnames(rdata)[14:19])
rdata=rdata[,c("type","weight_Co_responsive","r")]
gd<-rdata
colnames(gd)<-c("type","weight","r")
gd$weight=as.numeric(as.character(gd$weight))
library(pROC)
pproc(gd,rho=unique(gd$r),main="Co_responsive",filename = "Co_responsive_ROC.pdf",
      width = 4,height = 4)
###################################################################################################
##############################     simulated Composite     ########################################
###################################################################################################
simuComposite<-parLapply(cl,seq(-0.9,0.9,0.1),
                         function(r,simulate,modelSelection){
                           lapply(1:1000,function(x){
                             m<-rnorm(100)
                             e<-simulate(x=m,r=r)
                             g<-simulate(x=m,r=r)+simulate(x=e,r=r)
                             model1<-c(modelSelection(m,e,g),r=r,type="simulate")
                             m<-rnorm(100)
                             e<-rnorm(100)
                             g<-rnorm(100)
                             model2<-c(modelSelection(m,e,g),r=r,type="control")
                             rbind(model1,model2)
                           })
                         },simulate,modelSelection)
rdata<-c()
for (i in 1:length(simuComposite)) {
  x<-do.call(rbind,simuComposite[[i]])
  rdata<-rbind(rdata,x)
}
rdata=as.data.frame(rdata)
colnames(rdata)[14:19]<-gsub("AIC","weight",colnames(rdata)[14:19])
rdata=rdata[,c("type","weight_Composite","r")]
gd<-rdata
colnames(gd)<-c("type","weight","r")
gd$weight=as.numeric(as.character(gd$weight))
library(pROC)
pproc(gd,rho=unique(gd$r),main="Composite",filename = "Composite_ROC.pdf",
      width = 4,height = 4)
###################################################################################################
##############################     simulated Causal        ########################################
###################################################################################################
simuCausal<-parLapply(cl,seq(-0.9,0.9,0.1),
                      function(r,simulate,modelSelection){
                        lapply(1:1000,function(x){
                          m<-rnorm(100)
                          e<-simulate(x=m,r=r)
                          g<-simulate(x=e,r=r)
                          model1<-c(modelSelection(m,e,g),r=r,type="simulate")
                          m<-rnorm(100)
                          e<-rnorm(100)
                          g<-rnorm(100)
                          model2<-c(modelSelection(m,e,g),r=r,type="control")
                          rbind(model1,model2)
                        })
                      },simulate,modelSelection)
rdata<-c()
for (i in 1:length(simuCausal)) {
  x<-do.call(rbind,simuCausal[[i]])
  rdata<-rbind(rdata,x)
}
rdata=as.data.frame(rdata)
colnames(rdata)[14:19]<-gsub("AIC","weight",colnames(rdata)[14:19])
rdata=rdata[,c("type","weight_Causal","r")]
gd<-rdata
colnames(gd)<-c("type","weight","r")
gd$weight=as.numeric(as.character(gd$weight))
library(pROC)
pproc(gd,rho=unique(gd$r),main="Causal",filename = "Causal_ROC.pdf",
      width = 4,height = 4)
###################################################################################################
##############################     simulated Coordinate    ########################################
###################################################################################################
simuCoordinate<-parLapply(cl,seq(-0.9,0.9,0.1),
                      function(r,simulate,modelSelection){
                        lapply(1:1000,function(x){
                          m<-rnorm(100)
                          e<-rnorm(100)
                          g<-simulate(x=e,r=r)+simulate(x=m,r=r)
                          model1<-c(modelSelection(m,e,g),r=r,type="simulate")
                          m<-rnorm(100)
                          e<-rnorm(100)
                          g<-rnorm(100)
                          model2<-c(modelSelection(m,e,g),r=r,type="control")
                          rbind(model1,model2)
                        })
                      },simulate,modelSelection)
rdata<-c()
for (i in 1:length(simuCoordinate)) {
  x<-do.call(rbind,simuCoordinate[[i]])
  rdata<-rbind(rdata,x)
}
rdata=as.data.frame(rdata)
colnames(rdata)[14:19]<-gsub("AIC","weight",colnames(rdata)[14:19])
rdata=rdata[,c("type","weight_Coordinate","r")]
gd<-rdata
colnames(gd)<-c("type","weight","r")
gd$weight=as.numeric(as.character(gd$weight))
library(pROC)
pproc(gd,rho=unique(gd$r),main="Coordinate",filename = "Coordinate_ROC.pdf",
      width = 4,height = 4)
